// Check for 64-bit OS and define the Bits64_ macro required by Maya headers
//#include "defines.h"
//#ifdef X64
//#define Bits64_
//#endif

// Maya headers
#include <maya/MPxNode.h>
#include <maya/MFnNumericAttribute.h>
#include <maya/MFnTypedAttribute.h>
#include <maya/MFnStringData.h>
#include <maya/MFnMesh.h>
#include <maya/MFnMeshData.h>
#include <maya/MPointArray.h>
#include <maya/MPoint.h>
#include <maya/MFloatVector.h>

// V-Ray headers
#include "vraybase.h"
#include "maya_normaltest.h"
#include "vrayplugins.h"

// VRayNormalTest
MTypeId VRayNormalTest::id(0x000104F1); // Note: change this ID whenever a new plugin is created
MString VRayNormalTest::apiType("VRayNormalTest");
MString VRayNormalTest::classification("shader/surface/utility/:swatch/VRayMtlSwatchGen");

MObject VRayNormalTest::m_diffuse_color;

MObject VRayNormalTest::m_outColor;
MObject VRayNormalTest::m_outTransparency;
MObject VRayNormalTest::m_outApiType;
MObject VRayNormalTest::m_outApiClassification;

VRayNormalTest::VRayNormalTest() {
}

VRayNormalTest::~VRayNormalTest() {
}

void* VRayNormalTest::creator() {
    return new VRayNormalTest();
}

MStatus VRayNormalTest::initialize() {
    MFnNumericAttribute nAttr; 
    MFnTypedAttribute tAttr;
    MFnStringData strData;
    
    // Create and add output attributes
    // outColor
    m_outColor = nAttr.createColor("outColor", "oc");
    CHECK_MSTATUS(nAttr.setKeyable(false));
    CHECK_MSTATUS(nAttr.setStorable(false));
    CHECK_MSTATUS(nAttr.setReadable(true));
    CHECK_MSTATUS(nAttr.setWritable(false));
    CHECK_MSTATUS(addAttribute(m_outColor));

    // outTransparency
    m_outTransparency = nAttr.createColor("outTransparency", "ot");
    CHECK_MSTATUS(nAttr.setKeyable(false));
    CHECK_MSTATUS(nAttr.setStorable(false));
    CHECK_MSTATUS(nAttr.setReadable(true));
    CHECK_MSTATUS(nAttr.setWritable(false));
    CHECK_MSTATUS(addAttribute(m_outTransparency));

    // outApiType
    strData.create(VRayNormalTest::apiType);
    m_outApiType = tAttr.create("outApiType", "oat", MFnData::kString, strData.object());
    CHECK_MSTATUS(tAttr.setStorable(true));
    CHECK_MSTATUS(tAttr.setKeyable(false));
    CHECK_MSTATUS(tAttr.setHidden(true));
    CHECK_MSTATUS(addAttribute(m_outApiType));

    // outApiClassification
    strData.create(VRayNormalTest::classification);
    m_outApiClassification = tAttr.create("outApiClassification", "oac", MFnData::kString, strData.object());
    CHECK_MSTATUS(tAttr.setStorable(true));
    CHECK_MSTATUS(tAttr.setKeyable(false));
    CHECK_MSTATUS(tAttr.setHidden(true));
    CHECK_MSTATUS(addAttribute(m_outApiClassification));

    m_diffuse_color = nAttr.createColor("diffuseColor", "dcl");
    CHECK_MSTATUS(nAttr.setKeyable(true));
    CHECK_MSTATUS(nAttr.setStorable(true));
    CHECK_MSTATUS(nAttr.setReadable(true));
    CHECK_MSTATUS(nAttr.setWritable(true));
    CHECK_MSTATUS(nAttr.setDefault(0.f, 0.3f, 0.f));
    CHECK_MSTATUS(nAttr.setArray(true));
    CHECK_MSTATUS(addAttribute(m_diffuse_color));
    CHECK_MSTATUS(attributeAffects(m_diffuse_color, m_outColor));

    return MS::kSuccess;
}

// V-Ray does not call this method during rendering; it is here
// only to provide a level of compatibility with Maya.
MStatus VRayNormalTest::compute(const MPlug &plug, MDataBlock &block) {
    if (plug == m_outColor || plug.parent() == m_outColor) {
        MDataHandle outColorHandle = block.outputValue(m_outColor);
        MFloatVector &outColor = outColorHandle.asFloatVector();
        outColor = block.inputValue(m_diffuse_color).asFloatVector();
        outColorHandle.setClean();
        return MS::kSuccess;
    }
    else if (plug == m_outTransparency || plug.parent() == m_outTransparency) {
        MDataHandle outTransparencyHandle = block.outputValue(m_outTransparency);
        MFloatVector& outTransparency = outTransparencyHandle.asFloatVector();
        outTransparency = MFloatVector(0.0f, 0.0f, 0.0f); // MFloatVector(1.0f, 1.0f, 1.0f) - block.inputValue(m_opacityMap).asFloatVector();
        outTransparencyHandle.setClean();
        return MS::kSuccess;
    }
    else {
        return MS::kUnknownParameter;
    }
}
